@@ -1,7 +1,13 @@
 import collections
+import logging
 import base64
 import json
 
+import six
+
+
+log = logging.getLogger(__name__)
+
 
 class ConsulException(Exception):
     pass
@@ -158,6 +164,10 @@ class Consul(object):
             """
             Sets *key* to the given *value*.
 
+            *value* can either be None (useful for marking a key as a
+            directory) or any string type, including binary data (e.g. a
+            msgpack'd data structure)
+
             The optional *cas* parameter is used to turn the PUT into a
             Check-And-Set operation. This is very useful as it allows clients
             to build more complex syncronization primitives on top. If the
@@ -181,8 +191,11 @@ class Consul(object):
             The return value is simply either True or False. If False is
             returned, then the update has not taken place.
             """
-            # TODO: add acquire and release
             assert not key.startswith('/')
+            assert value is None or \
+                isinstance(value, (six.string_types, six.binary_type)), \
+                "value should be None or a string / binary data"
+
             params = {}
             if cas is not None:
                 params['cas'] = cas
@@ -237,6 +250,7 @@ class Consul(object):
         def __init__(self, agent):
             self.agent = agent
             self.service = Consul.Agent.Service(agent)
+            self.check = Consul.Agent.Check(agent)
 
         def self(self):
             """
@@ -259,6 +273,30 @@ class Consul(object):
             return self.agent.http.get(
                 lambda x: json.loads(x.body), '/v1/agent/services')
 
+        def checks(self):
+            """
+            Returns all the checks that are registered with the local agent.
+            These checks were either provided through configuration files, or
+            added dynamically using the HTTP API. Similar to services,
+            the checks known by the agent may be different than those
+            reported by the Catalog. This is usually due to changes being made
+            while there is no leader elected. The agent performs active
+            anti-entropy, so in most situations everything will be in sync
+            within a few seconds.
+            """
+            return self.agent.http.get(
+                lambda x: json.loads(x.body), '/v1/agent/checks')
+
+        def members(self):
+            """
+            Returns all the members that this agent currently sees. This may
+            vary by agent, use the nodes api of Catalog to retrieve a cluster
+            wide consistent view of members
+            """
+            # TODO: add wan flag
+            return self.agent.http.get(
+                lambda x: json.loads(x.body), '/v1/agent/members')
+
         class Service(object):
             def __init__(self, agent):
                 self.agent = agent
@@ -309,6 +347,116 @@ class Consul(object):
                     lambda x: x.code == 200,
                     '/v1/agent/service/deregister/%s' % service_id)
 
+        class Check(object):
+            def __init__(self, agent):
+                self.agent = agent
+
+            def register(
+                    self,
+                    name,
+                    check_id=None,
+                    script=None,
+                    interval=None,
+                    ttl=None,
+                    notes=None):
+                """
+                Register a new check with the local agent. More documentation
+                on checks can be found `here
+                <http://www.consul.io/docs/agent/checks.html>`_.
+
+                *name* is the name of the check.
+
+                If the optional *check_id* is not provided it is set to *name*.
+                *check_id* must be unique for this agent.
+
+                There are two ways to define a check, either with a *script*
+                and an *interval* or with a *ttl* timeout. If a *script* is
+                supplied then the *interval* is expected and the *ttl* should
+                be absent. For a *ttl* check, the *script* and *interval*
+                should not be supplied.
+
+                *notes* is not used by Consul, and is meant to be human
+                readable.
+
+                Returns *True* on success.
+                """
+                payload = {'name': name}
+                if check_id:
+                    payload['id'] = check_id
+
+                assert script or ttl, 'Either script or ttl is required'
+
+                if script:
+                    assert interval, 'Interval required for script check'
+                    assert not ttl, 'ttl not used with script based check'
+                    payload['script'] = script
+                    payload['interval'] = interval
+
+                if ttl:
+                    assert not (interval or script), \
+                        'Interval and script not required for ttl check'
+                    payload['ttl'] = ttl
+
+                if notes:
+                    payload['note'] = notes
+
+                return self.agent.http.put(
+                    lambda x: x.code == 200,
+                    '/v1/agent/check/register',
+                    data=json.dumps(payload))
+
+            def deregister(self, check_id):
+                """
+                Remove a check from the local agent.
+                """
+                return self.agent.http.get(
+                    lambda x: x.code == 200,
+                    '/v1/agent/check/deregister/%s' % check_id)
+
+            def ttl_pass(self, check_id, notes=None):
+                """
+                Mark a ttl based check as passing. Optional notes can be
+                attached to describe the status of the check.
+                """
+                params = {}
+                if notes:
+                    params['note'] = notes
+
+                return self.agent.http.get(
+                    lambda x: x.code == 200,
+                    '/v1/agent/check/pass/%s' % check_id,
+                    params=params)
+
+            def ttl_fail(self, check_id, notes=None):
+                """
+                Mark a ttl based check as failing. Optional notes can be
+                attached to describe why check is failing. The status of the
+                check will be set to critical and the ttl clock will be reset.
+                """
+                params = {}
+                if notes:
+                    params['note'] = notes
+
+                return self.agent.http.get(
+                    lambda x: x.code == 200,
+                    '/v1/agent/check/fail/%s' % check_id,
+                    params=params)
+
+            def ttl_warn(self, check_id, notes=None):
+                """
+                Mark a ttl based check with warning. Optional notes can be
+                attached to describe the warning. The status of the
+                check will be set to warn and the ttl clock will be reset.
+                """
+                params = {}
+                if notes:
+                    params['note'] = notes
+
+                return self.agent.http.get(
+                    lambda x: x.code == 200,
+                    '/v1/agent/check/warn/%s' % check_id,
+                    params=params)
+
     class Catalog(object):
         def __init__(self, agent):
             self.agent = agent
@@ -627,11 +775,14 @@ class Consul(object):
         class Check(object):
             def __init__(self, agent):
                 self.agent = agent
+                self.warned = False
 
             def ttl_pass(self, check_id):
                 """
                 Mark a local TTL check as passing.
                 """
+                if not self.warned:
+                    log.warn('DEPRECATED: update call to agent.check.ttl_pass')
                 return self.agent.http.get(
                     lambda x: x.code == 200,
                     '/v1/agent/check/pass/%s' % check_id)
